from netbox.navigation import *
from netbox.navigation.menu import MENUS


MIGRATION = Menu(
    label="Migration",
    icon_class="mdi mdi-cog-transfer",
    groups=(
        MenuGroup(
            label="SOP-Migration",
            items=(
                MenuItem(
                    link=f"plugins:sop_migration:sopmigration_list",
                    link_text="Migrations",
                    permissions=[f"sop_migration.view_sopmigration"],
                    buttons=(
                        MenuItemButton(
                            link=f"plugins:sop_migration:sopmigration_add",
                            title="Add",
                            icon_class="mdi mdi-plus-thick",
                            permissions=[f"sop_migration.add_sopmigration"],
                        ),
                    ),
                ),
            ),
        ),
    ),
)


MENUS.append(MIGRATION)



from netbox.plugins import PluginConfig


class SopMigrationConfig(PluginConfig):
    name = "sop_migration"
    verbose_name = "SOP Migration"
    description = "Pin your migrations to any NetBox model instance"
    version = "0.0.6"
    author = "Leorevoir"
    author_email = "leo.quinzler@epitech.eu"
    base_url = "sop-migration"
    min_version = "4.1.0"


config = SopMigrationConfig



from netbox.api.metadata import ContentTypeMetadata
from netbox.api.viewsets import NetBoxModelViewSet

from sop_migration.api.serializers import SopMigrationSerializer
from sop_migration.models import SopMigration


__all__ = ("SopMigrationViewSet",)


class SopMigrationViewSet(NetBoxModelViewSet):

    metadata_class = ContentTypeMetadata
    queryset = SopMigration.objects.all()
    serializer_class = SopMigrationSerializer



import django_tables2 as tables
from netbox.tables import NetBoxTable
from sop_migration.models import SopMigration


__all__ = ("SopMigrationTable",)


class SopMigrationTable(NetBoxTable):

    object_type = tables.Column(verbose_name="Object (Type)", linkify=True)
    object_id = tables.Column(verbose_name="Object (ID)", linkify=True)

    class Meta(NetBoxTable.Meta):
        model = SopMigration
        fields = (
            "actions",
            "pk",
            "id",
            "object_type",
            "object_id",
            "date",
            "cut",
            "impact",
            "state",
            "description",
            "comments",
        )
        default_columns = (
            "actions",
            "pk",
            " object_type",
            "object_id",
            "date",
            "description",
        )

    def render_object_id(self, record):
        return record.object_type.get_object_for_this_type(pk=record.object_id)



from django.urls import path

from netbox.views.generic import ObjectChangeLogView, ObjectJournalView

from sop_migration.views import *
from sop_migration.models import *


urlpatterns = (
    path("migration/", SopMigrationListView.as_view(), name="sopmigration_list"),
    path("migration/add/", SopMigrationEditView.as_view(), name="sopmigration_add"),
    path(
        "migration/edit/<int:pk>/",
        SopMigrationEditView.as_view(),
        name="sopmigration_edit",
    ),
    path("migration/<int:pk>/", SopMigrationView.as_view(), name="sopmigration_detail"),
    path(
        "migration/delete/<int:pk>/",
        SopMigrationDeleteView.as_view(),
        name="sopmigration_delete",
    ),
    path(
        "migration/changelog/<int:pk>/",
        ObjectChangeLogView.as_view(),
        name="sopmigration_changelog",
        kwargs={"model": SopMigration},
    ),
    path(
        "migration/journal/<int:pk>/",
        ObjectJournalView.as_view(),
        name="sopmigration_journal",
        kwargs={"model": SopMigration},
    ),
)



from rest_framework import serializers

from core.models import ObjectType
from netbox.api.fields import ContentTypeField
from netbox.api.serializers import NetBoxModelSerializer

from sop_migration.models import SopMigration


__all__ = ("SopMigrationSerializer",)


class SopMigrationSerializer(NetBoxModelSerializer):

    url = serializers.HyperlinkedIdentityField(
        view_name="plugins-api:sop_migration-api:sopmigration-detail"
    )
    object_type = ContentTypeField(queryset=ObjectType.objects.all())

    class Meta:
        model = SopMigration
        fields = (
            "id",
            "url",
            "display",
            "object_type",
            "object_id",
            "date",
            "cut",
            "impact",
            "state",
            "description",
            "comments",
            "created",
            "last_updated",
        )
        brief_fields = ("id", "url", "display", "date", "description")



from django.db.models import Q
from core.models.contenttypes import ObjectType
from netbox.filtersets import NetBoxModelFilterSet
from sop_migration.models import SopMigration


__all__ = ("SopMigrationFilterSet",)


class SopMigrationFilterSet(NetBoxModelFilterSet):

    class Meta:
        model = SopMigration
        fields = ("id", "object_type", "object_id", "date", "state", "impact")

    def search_object_name(self, queryset, value):
        valid_ids: list[int] = []

        for instance in queryset:
            obj = ObjectType.objects.get(pk=instance.object_type_id)
            target = obj.get_object_for_this_type(pk=instance.object_id)

            if value in target.__str__():
                valid_ids.append(instance.id)
                continue

            elif value in str(instance.date.__str__()):
                valid_ids.append(instance.id)
                continue

        return queryset.filter(id__in=valid_ids)

    def search(self, queryset, name, value):
        if not value.strip():
            return queryset

        query = self.search_object_name(queryset, value)
        if query is not None:
            return query

        return queryset.filter(
            Q(object_type__id__icontains=value)
            | Q(object_id__icontains=value)
            | Q(description__icontains=value)
            | Q(comments__icontains=value)
        )



from netbox.search import SearchIndex, register_search

from sop_migration.models import SopMigration


__all__ = ("SopMigrationSearchIndex",)


@register_search
class SopMigrationSearchIndex(SearchIndex):

    model = SopMigration
    fields = (
        ("object_type", 100),
        ("object_id", 100),
        ("date", 100),
        ("cut", 1000),
        ("impact", 1000),
        ("state", 1000),
        ("description", 500),
        ("comments", 500),
    )



from netbox.api.routers import NetBoxRouter

from sop_migration.api.views import SopMigrationViewSet


app_name = "sop_migrations"

router = NetBoxRouter()
router.register("sop-migrations", SopMigrationViewSet)

urlpatterns = router.urls



import logging

from django.apps import apps
from django.conf import settings
from django.http import HttpResponse

from netbox.plugins import PluginTemplateExtension

from sop_migration.models import SopMigration


ALLOWED_POSITIONS = ["left_page", "right_page"]


# overrides NetBox PluginTemplateExtension method to display
# the right panel according to PLUGINS_CONFIG in configuration.py
def create_new_panel(self) -> HttpResponse:

    def get_extra_context() -> dict:
        return {"migration": SopMigration}

    return self.render(self.template_name, get_extra_context())


class SopMigrationDashboard:

    template_name = "sop_migration/panel.html"
    settings = settings.PLUGINS_CONFIG.get("sop_migration", {})

    def __init__(self) -> None:
        self.extensions = self.get_display_extensions()

    def get_display_position(self, model, _display) -> str | None:

        if exists := _display.get(model):

            if exists not in ALLOWED_POSITIONS:
                return None

            return exists

        return None

    def get_display_extensions(self) -> list | None:

        extensions = []
        _display = self.settings.get("display")

        if _display is None:
            return None

        # check if display is in intended format {model:position}
        if not isinstance(_display, dict):
            logging.error(f'Invalid syntax "{_display}" must be a dict.')
            return None

        # iterate existing application models
        for app_model in apps.get_models():

            # format object into something else than <class>
            model = app_model._meta.model_name

            if model not in _display:
                continue

            # get and check position in {model:position}
            position = self.get_display_position(model, _display)
            if position is None:
                logging.error(
                    f'Invalid position "{position}" is not a valid choice between {ALLOWED_POSITIONS}.'
                )
                return None

            new_class = type(
                f"{model}_migration_extension",
                (PluginTemplateExtension,),
                {
                    "template_name": self.template_name,
                    "model": f"{app_model._meta.app_label}.{model}",
                    position: create_new_panel,
                },
            )
            extensions.append(new_class)

        return extensions

    def push(self) -> list[object] | None:
        return self.extensions or None


template_extensions = SopMigrationDashboard().push()



from django.forms import DateTimeField, IntegerField, ChoiceField
from django.test.utils import require_jinja2

from core.models.contenttypes import ObjectType
from utilities.forms.utils import add_blank_choice
from utilities.forms.fields import CommentField, DynamicModelMultipleChoiceField
from utilities.forms.widgets import DateTimePicker
from utilities.forms.widgets.apiselect import APISelectMultiple
from netbox.forms import NetBoxModelForm, NetBoxModelFilterSetForm

from sop_migration.models import MigrationStateChoices, SopMigration


__all__ = ("SopMigrationForm", "SopMigrationFilterSetForm")


class SopMigrationForm(NetBoxModelForm):

    date = DateTimeField(label="Migration date", widget=DateTimePicker())
    comments = CommentField()

    class Meta:
        model = SopMigration
        fields = [
            "object_type",
            "object_id",
            "date",
            "cut",
            "impact",
            "state",
            "description",
            "comments",
        ]

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

        if "tags" in self.fields:
            del self.fields["tags"]


class SopMigrationFilterSetForm(NetBoxModelFilterSetForm):
    model = SopMigration
    object_type = DynamicModelMultipleChoiceField(
        queryset=ObjectType.objects.all(),
        required=False,
        label="Object (Type)",
        widget=APISelectMultiple(api_url=f"/api/extras/object-types/"),
    )
    date = DateTimeField(
        required=False, label="Migration date", widget=DateTimePicker()
    )
    cut = IntegerField(required=False)
    state = ChoiceField(choices=add_blank_choice(MigrationStateChoices), required=False)



# Generated by Django 5.0.9 on 2024-11-27 15:03

import django.db.models.deletion
import taggit.managers
import utilities.json
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('core', '0012_job_object_type_optional'),
        ('extras', '0121_customfield_related_object_filter'),
    ]

    operations = [
        migrations.CreateModel(
            name='SopMigration',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False)),
                ('created', models.DateTimeField(auto_now_add=True, null=True)),
                ('last_updated', models.DateTimeField(auto_now=True, null=True)),
                ('custom_field_data', models.JSONField(blank=True, default=dict, encoder=utilities.json.CustomFieldJSONEncoder)),
                ('object_id', models.PositiveBigIntegerField()),
                ('date', models.DateTimeField()),
                ('cut', models.PositiveBigIntegerField()),
                ('impact', models.PositiveBigIntegerField()),
                ('state', models.CharField()),
                ('description', models.CharField(blank=True, max_length=200)),
                ('comments', models.TextField(blank=True)),
                ('object_type', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='core.objecttype')),
                ('tags', taggit.managers.TaggableManager(through='extras.TaggedItem', to='extras.Tag')),
            ],
            options={
                'verbose_name': 'Migration',
                'verbose_name_plural': 'Migrations',
                'ordering': ('object_type', 'pk', 'date'),
                'abstract': False,
            },
        ),
    ]



from django.urls import reverse
from django.db import models

from utilities.choices import ChoiceSet
from core.models.contenttypes import ObjectType
from netbox.models import NetBoxModel


__all__ = ("SopMigration",)


class MigrationStateChoices(ChoiceSet):

    CHOICES = (
        ("unknown", "Unknown", "gray"),
        ("choix1", "Choix1", "green"),
        ("choix2", "Choix2", "red"),
    )


class SopMigration(NetBoxModel):

    # object to pin (type + ID)
    object_type = models.ForeignKey(
        to=ObjectType, on_delete=models.CASCADE, verbose_name="Object (Type)"
    )
    object_id = models.PositiveBigIntegerField(verbose_name="Object (ID)")

    # attributes
    date = models.DateTimeField(verbose_name="Migration date")
    cut = models.PositiveBigIntegerField()
    impact = models.PositiveBigIntegerField()
    state = models.CharField(choices=MigrationStateChoices)
    description = models.CharField(max_length=200, blank=True)
    comments = models.TextField(blank=True)

    class Meta(NetBoxModel.Meta):
        ordering = ("object_type", "pk", "date")
        verbose_name = "Migration"
        verbose_name_plural = "Migrations"

    def __str__(self) -> str:
        try:
            app = self.object_type.app_label.upper()
            obj = self.object_type.get_object_for_this_type(id=self.object_id)
            return f"{app} | {obj} Migration - {self.date}"
        except:
            return "Migration"

    def get_absolute_url(self):
        return reverse("plugins:sop_migration:sopmigration_detail", args=[self.pk])



from setuptools import setup, find_packages


setup(
    name="sop-migration",
    version="0.0.6",
    packages=find_packages(),
    include_package_data=True,
    description="Pin your migrations to any NetBox model instance",
    author="Leorevoir",
    author_email="leo.quinzler@epitech.eu",
    classifiers=[
        "Framework :: Django",
        "Programming Language :: Python :: 3",
    ],
    zip_safe=False,
)



from django.db.models.query import InstanceCheckMeta
from netbox.views import generic

from sop_migration.models import SopMigration
from sop_migration.forms import SopMigrationForm, SopMigrationFilterSetForm
from sop_migration.tables import SopMigrationTable
from sop_migration.filtersets import SopMigrationFilterSet


__all__ = (
    "SopMigrationView",
    "SopMigrationListView",
    "SopMigrationEditView",
    "SopMigrationDeleteView",
)


class SopMigrationView(generic.ObjectView):

    queryset = SopMigration.objects.all()

    def get_extra_context(self, request, instance) -> dict:
        context = super().get_extra_context(request, instance)
        pk = instance.object_id
        obj = instance.object_type

        # get model by ID
        context["model_name"] = obj.get_object_for_this_type(pk=pk)

        return {**context}


class SopMigrationListView(generic.ObjectListView):

    queryset = SopMigration.objects.all()
    table = SopMigrationTable
    filterset = SopMigrationFilterSet
    filterset_form = SopMigrationFilterSetForm


class SopMigrationEditView(generic.ObjectEditView):

    queryset = SopMigration.objects.all()
    form = SopMigrationForm

    def get_extra_context(self, request, instance):
        context: dict = super().get_extra_context(request, instance)

        return context

    def get_extra_addanother_params(self, request):

        return {"object_type": self.object_type, "object_id": self.object_id}

    def get(self, request, *args, **kwargs):
        self.object_type = request.GET.get("object_type")
        self.object_id = request.GET.get("object_id")

        return super().get(request, *args, **kwargs)


class SopMigrationDeleteView(generic.ObjectDeleteView):

    queryset = SopMigration.objects.all()
